<?php

namespace Drupal\page_not_found_files_reports\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for Page Not Found files routes.
 */
class FilesLogController extends ControllerBase {

  /**
   * The database connector.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $db;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The configuration entity.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Construct function.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    Connection $connection,
    RequestStack $request_stack
    ) {
    $this->config = $config_factory->getEditable('pnf_files_reports.settings');
    $this->db = $connection;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('database'),
      $container->get('request_stack')
    );
  }

  /**
   * Shows the 404 log messages for all file types.
   *
   * @return array
   *   A build array in the format expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   */
  public function filesLog() {
    $header = [
      ['data' => $this->t('File URL'), 'field' => 'message'],
    ];
    $rows = [];
    $feed_url = $this->config->get('page_url');
    $host = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
    if ($feed_url) {
      $build['help'] = [
        '#type' => 'markup',
        '#markup' => '<h5>Configured link empty files report</h5>',

      ];
      // Get URL content using file_get_contents function and store content in
      // php variable.
      $urlContent = file_get_contents($feed_url);

      if ($urlContent) {
        $dom = new \DOMDocument();
        @$dom->loadHTML($urlContent);
        $xpath = new \DOMXPath($dom);
        $imgs = $xpath->evaluate("/html/body//img");

        for ($i = 0; $i < $imgs->length; $i++) {
          $img = $imgs->item($i);
          $src = $img->getAttribute('src');

          // Sanitize the extracted url inside loop.
          $url = filter_var($src, FILTER_SANITIZE_URL);
          if (filter_var($url, FILTER_VALIDATE_URL) === FALSE) {

            $url = $host . $url;
          }
          // Use get_headers() function.
          $headers = @get_headers($url);
          if (!($headers && strpos($headers[0], '200'))) {
            // $url." URL not Exist";
            $rows[] = [$url];
          }

        }
      }
    }
    else {
      $build['help'] = [
        '#type' => 'markup',
        '#markup' => '<h4>Sitemap.xml pages empty files report</h4>',

      ];
      // Sitemap url or sitemap file.
      $sitemap = $host . '/sitemap.xml';

      // Get sitemap content.
      $content = file_get_contents($sitemap);

      // Parse the sitemap content to object.
      $xml = simplexml_load_string($content);
      if ($xml) {
        // Retrieve properties from the sitemap object.
        $urlElement = $xml->url;

        $rows = [];
        foreach ($xml->url as $urlElement) {
          // Get properties
          // store URL in php variable.
          $fetchurl = $urlElement->loc;

          // Get URL content using file_get_contents function and store content
          // in php variable.
          $urlContent = file_get_contents($fetchurl);

          $dom = new \DOMDocument();
          @$dom->loadHTML($urlContent);
          $xpath = new \DOMXPath($dom);
          $imgs = $xpath->evaluate("/html/body//img");

          for ($i = 0; $i < $imgs->length; $i++) {
            $img = $imgs->item($i);
            $src = $img->getAttribute('src');

            // Sanitize the extracted url inside loop.
            $url = filter_var($src, FILTER_SANITIZE_URL);
            if (filter_var($url, FILTER_VALIDATE_URL) === FALSE) {

              $url = $host . $url;
            }
            // Use get_headers() function.
            $headers = @get_headers($url);
            if (!($headers && strpos($headers[0], '200'))) {
              // $url." URL not Exist";
              $rows[] = [$url];
            }

          }
        }

      }

    }

    $build['dblog_top_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No logs available.'),
    ];
    $build['dblog_top_pager'] = ['#type' => 'pager'];

    return $build;
  }

}
